import{s as it,n as nt,o as at}from"../chunks/scheduler.048ce5c2.js";import{S as ot,i as st,g as s,s as a,r as h,A as lt,h as l,f as i,c as o,j as et,u,x as d,k as tt,y as dt,a as n,v as f,d as g,t as m,w as c}from"../chunks/index.2633050f.js";import{H as p}from"../chunks/Heading.bb054f8c.js";function rt(Pe){let r,V,Y,J,b,X,w,De="Welcome to Unit 2 of the Hugging Face Diffusion Models Course! In this unit, you will learn how to use and adapt pre-trained diffusion models in new ways. You will also see how we can create diffusion models that take additional inputs as <strong>conditioning</strong> to control the generation process.",Z,v,ee,x,Ie="Here are the steps for this unit:",te,y,Se='<li>Make sure you’ve <a href="https://huggingface.us17.list-manage.com/subscribe?u=7f57e683fa28b51bfc493d048&amp;id=ef963b4162" rel="nofollow">signed up for this course</a> so that you can be notified when new material is released.</li> <li>Read through the material below for an overview of the key ideas of this unit.</li> <li>Check out the <em><strong>Fine-tuning and Guidance</strong></em> notebook to fine-tune an existing diffusion model on a new dataset using the 🤗 Diffusers library and to modify the sampling procedure using guidance.</li> <li>Follow the example in the notebook to share a Gradio demo for your custom model.</li> <li>(Optional) Check out the <em><strong>Class-conditioned Diffusion Model Example</strong></em> notebook to see how we can add additional control to the generation process.</li> <li>(Optional) Check out <a href="https://www.youtube.com/watch?v=mY20iKOQ2zw" rel="nofollow">this video</a> for an informal run-through of the material in this unit.</li>',ie,_,Ge=':loudspeaker: Don’t forget to join the <a href="https://huggingface.co/join/discord" rel="nofollow">Discord</a>, where you can discuss the material and share what you’ve made in the <code>#diffusion-models-class</code> channel.',ne,k,ae,T,Fe="As you may have seen in Unit 1, training diffusion models from scratch can be time-consuming! Especially as we push to higher resolutions, the time and data required to train a model from scratch can become impractical. Fortunately, there is a solution: begin with a model that has already been trained! This way we start from a model that has already learned to denoise images of some kind, and the hope is that this provides a better starting point than beginning from a randomly initialized model.",oe,$,Ue='<img src="https://api.wandb.ai/files/johnowhitaker/dm_finetune/2upaa341/media/images/Sample%20generations_501_d980e7fe082aec0dfc49.png" alt="Example images generated with a model trained on LSUN Bedrooms and fine-tuned for 500 steps on WikiArt"/>',se,C,je='Fine-tuning typically works best if the new data somewhat resembles the base model’s original training data (for example, beginning with a model trained on faces is probably a good idea if you’re trying to generate cartoon faces) but surprisingly the benefits persist even if the domain is changed quite drastically. The image above is generated from a <a href="https://huggingface.co/google/ddpm-bedroom-256" rel="nofollow">model trained on the LSUN Bedrooms dataset</a> and fine-tuned for 500 steps on <a href="https://huggingface.co/datasets/huggan/wikiart" rel="nofollow">the WikiArt dataset</a>. The <a href="https://github.com/huggingface/diffusion-models-class/blob/main/unit2/finetune_model.py" rel="nofollow">training script</a> is included for reference alongside the notebooks for this unit.',le,M,de,L,Ee="Unconditional models don’t give much control over what is generated. We can train a conditional model (more on that in the next section) that takes additional inputs to help steer the generation process, but what if we already have a trained unconditional model we’d like to use? Enter guidance, a process by which the model predictions at each step in the generation process are evaluated against some guidance function and modified such that the final generated image is more to our liking.",re,H,Ae='<img src="https://huggingface.co/datasets/huggingface-course/documentation-images/resolve/main/diffusion-course/guidance_eg.png" alt="guidance example image"/>',he,P,qe="This guidance function can be almost anything, making this a powerful technique! In the notebook, we build up from a simple example (controlling the color, as illustrated in the example output above) to one utilizing a powerful pre-trained model called CLIP which lets us guide generation based on a text description.",ue,D,fe,I,ze="Guidance is a great way to get some additional mileage from an unconditional diffusion model, but if we have additional information (such as a class label or an image caption) available during training then we can also feed this to the model for it to use as it makes its predictions. In doing so, we create a <strong>conditional</strong> model, which we can control at inference time by controlling what is fed in as conditioning. The notebook shows an example of a class-conditioned model which learns to generate images according to a class label.",ge,S,Oe='<img src="https://huggingface.co/datasets/huggingface-course/documentation-images/resolve/main/diffusion-course/conditional_digit_generation.png" alt="conditioning example"/>',me,G,Ne="There are a number of ways to pass in this conditioning information, such as",ce,F,Ke='<li>Feeding it in as additional channels in the input to the UNet. This is often used when the conditioning information is the same shape as the image, such as a segmentation mask, a depth map or a blurry version of the image (in the case of a restoration/superresolution model). It does work for other types of conditioning too. For example, in the notebook, the class label is mapped to an embedding and then expanded to be the same width and height as the input image so that it can be fed in as additional channels.</li> <li>Creating an embedding and then projecting it down to a size that matches the number of channels at the output of one or more internal layers of the UNet, and then adding it to those outputs. This is how the timestep conditioning is handled, for example. The output of each Resnet block has a projected timestep embedding added to it. This is useful when you have a vector such as a CLIP image embedding as your conditioning information. A notable example is the <a href="https://huggingface.co/spaces/lambdalabs/stable-diffusion-image-variations" rel="nofollow">‘Image Variations’ version of Stable Diffusion</a> which does exactly this.</li> <li>Adding cross-attention layers that can ‘attend’ to a sequence passed in as conditioning. This is most useful when the conditioning is in the form of some text - the text is mapped to a sequence of embeddings using a transformer model, and then cross-attention layers in the UNet are used to incorporate this information into the denoising path. We’ll see this in action in Unit 3 as we examine how Stable Diffusion handles text conditioning.</li>',pe,U,be,j,Re='<thead><tr><th align="left">Chapter</th> <th align="left">Colab</th> <th align="left">Kaggle</th> <th align="left">Gradient</th> <th align="left">Studio Lab</th></tr></thead> <tbody><tr><td align="left">Fine-tuning and Guidance</td> <td align="left"><a href="https://colab.research.google.com/github/huggingface/diffusion-models-class/blob/main/unit2/01_finetuning_and_guidance.ipynb" rel="nofollow"><img src="https://colab.research.google.com/assets/colab-badge.svg" alt="Open In Colab"/></a></td> <td align="left"><a href="https://kaggle.com/kernels/welcome?src=https://github.com/huggingface/diffusion-models-class/blob/main/unit2/01_finetuning_and_guidance.ipynb" rel="nofollow"><img src="https://kaggle.com/static/images/open-in-kaggle.svg" alt="Kaggle"/></a></td> <td align="left"><a href="https://console.paperspace.com/github/huggingface/diffusion-models-class/blob/main/unit2/01_finetuning_and_guidance.ipynb" rel="nofollow"><img src="https://assets.paperspace.io/img/gradient-badge.svg" alt="Gradient"/></a></td> <td align="left"><a href="https://studiolab.sagemaker.aws/import/github/huggingface/diffusion-models-class/blob/main/unit2/01_finetuning_and_guidance.ipynb" rel="nofollow"><img src="https://studiolab.sagemaker.aws/studiolab.svg" alt="Open In SageMaker Studio Lab"/></a></td></tr> <tr><td align="left">Class-conditioned Diffusion Model Example</td> <td align="left"><a href="https://colab.research.google.com/github/huggingface/diffusion-models-class/blob/main/unit2/02_class_conditioned_diffusion_model_example.ipynb" rel="nofollow"><img src="https://colab.research.google.com/assets/colab-badge.svg" alt="Open In Colab"/></a></td> <td align="left"><a href="https://kaggle.com/kernels/welcome?src=https://github.com/huggingface/diffusion-models-class/blob/main/unit2/02_class_conditioned_diffusion_model_example.ipynb" rel="nofollow"><img src="https://kaggle.com/static/images/open-in-kaggle.svg" alt="Kaggle"/></a></td> <td align="left"><a href="https://console.paperspace.com/github/huggingface/diffusion-models-class/blob/main/unit2/02_class_conditioned_diffusion_model_example.ipynb" rel="nofollow"><img src="https://assets.paperspace.io/img/gradient-badge.svg" alt="Gradient"/></a></td> <td align="left"><a href="https://studiolab.sagemaker.aws/import/github/huggingface/diffusion-models-class/blob/main/unit2/02_class_conditioned_diffusion_model_example.ipynb" rel="nofollow"><img src="https://studiolab.sagemaker.aws/studiolab.svg" alt="Open In SageMaker Studio Lab"/></a></td></tr></tbody>',we,E,We="At this point, you know enough to get started with the accompanying notebooks! Open them in your platform of choice using the links above. Fine-tuning is quite computationally intensive, so if you’re using Kaggle or Google Colab make sure you set the runtime type to ‘GPU’ for the best results.",ve,A,Be='The bulk of the material is in <em><strong>Fine-tuning and Guidance</strong></em>, where we explore these two topics through worked examples. The notebook shows how you can fine-tune an existing model on new data, add guidance, and share the result as a Gradio demo. There is an accompanying script (<a href="https://github.com/huggingface/diffusion-models-class/blob/main/unit2/finetune_model.py" rel="nofollow">finetune_model.py</a>) that makes it easy to experiment with different fine-tuning settings, and an <a href="https://huggingface.co/spaces/johnowhitaker/color-guided-wikiart-diffusion" rel="nofollow">example space</a> that you can use as a template for sharing your own demo on 🤗 Spaces.',xe,q,Ye="In the <em><strong>Class-conditioned Diffusion Model Example</strong></em>, we show a brief worked example of creating a diffusion model conditioned on class labels using the MNIST dataset. The focus is on demonstrating the core idea as simply as possible: by giving the model extra information about what it is supposed to be denoising, we can later control what kinds of images are generated at inference time.",ye,z,_e,O,Qe="Following the examples in the <em><strong>Fine-tuning and Guidance</strong></em> notebook, fine-tune your own model or pick an existing model and create a Gradio demo to showcase your new guidance skills. Don’t forget to share your demo on Discord, Twitter etc so we can admire your work!",ke,N,Te,K,Ve='<a href="https://arxiv.org/abs/2010.02502" rel="nofollow">Denoising Diffusion Implicit Models</a> - Introduced the DDIM sampling method (used by DDIMScheduler)',$e,R,Je='<a href="https://arxiv.org/abs/2112.10741" rel="nofollow">GLIDE: Towards Photorealistic Image Generation and Editing with Text-Guided Diffusion Models</a> - Introduced methods for conditioning diffusion models on text',Ce,W,Xe='<a href="https://arxiv.org/abs/2211.01324" rel="nofollow">eDiffi: Text-to-Image Diffusion Models with an Ensemble of Expert Denoisers</a> - Shows how many different kinds of conditioning can be used together to give even more control over the kinds of samples generated',Me,B,Ze="Found more great resources? Let us know and we’ll add them to this list.",Le,Q,He;return b=new p({props:{title:"Unit 2: Fine-Tuning, Guidance and Conditioning",local:"unit-2-fine-tuning-guidance-and-conditioning",headingTag:"h1"}}),v=new p({props:{title:"Start this Unit :rocket:",local:"start-this-unit-rocket",headingTag:"h2"}}),k=new p({props:{title:"Fine-Tuning",local:"fine-tuning",headingTag:"h2"}}),M=new p({props:{title:"Guidance",local:"guidance",headingTag:"h2"}}),D=new p({props:{title:"Conditioning",local:"conditioning",headingTag:"h2"}}),U=new p({props:{title:"Hands-On Notebook",local:"hands-on-notebook",headingTag:"h2"}}),z=new p({props:{title:"Project Time",local:"project-time",headingTag:"h2"}}),N=new p({props:{title:"Some Additional Resources",local:"some-additional-resources",headingTag:"h2"}}),{c(){r=s("meta"),V=a(),Y=s("p"),J=a(),h(b.$$.fragment),X=a(),w=s("p"),w.innerHTML=De,Z=a(),h(v.$$.fragment),ee=a(),x=s("p"),x.textContent=Ie,te=a(),y=s("ul"),y.innerHTML=Se,ie=a(),_=s("p"),_.innerHTML=Ge,ne=a(),h(k.$$.fragment),ae=a(),T=s("p"),T.textContent=Fe,oe=a(),$=s("p"),$.innerHTML=Ue,se=a(),C=s("p"),C.innerHTML=je,le=a(),h(M.$$.fragment),de=a(),L=s("p"),L.textContent=Ee,re=a(),H=s("p"),H.innerHTML=Ae,he=a(),P=s("p"),P.textContent=qe,ue=a(),h(D.$$.fragment),fe=a(),I=s("p"),I.innerHTML=ze,ge=a(),S=s("p"),S.innerHTML=Oe,me=a(),G=s("p"),G.textContent=Ne,ce=a(),F=s("ul"),F.innerHTML=Ke,pe=a(),h(U.$$.fragment),be=a(),j=s("table"),j.innerHTML=Re,we=a(),E=s("p"),E.textContent=We,ve=a(),A=s("p"),A.innerHTML=Be,xe=a(),q=s("p"),q.innerHTML=Ye,ye=a(),h(z.$$.fragment),_e=a(),O=s("p"),O.innerHTML=Qe,ke=a(),h(N.$$.fragment),Te=a(),K=s("p"),K.innerHTML=Ve,$e=a(),R=s("p"),R.innerHTML=Je,Ce=a(),W=s("p"),W.innerHTML=Xe,Me=a(),B=s("p"),B.textContent=Ze,Le=a(),Q=s("p"),this.h()},l(e){const t=lt("svelte-u9bgzb",document.head);r=l(t,"META",{name:!0,content:!0}),t.forEach(i),V=o(e),Y=l(e,"P",{}),et(Y).forEach(i),J=o(e),u(b.$$.fragment,e),X=o(e),w=l(e,"P",{"data-svelte-h":!0}),d(w)!=="svelte-u6zj26"&&(w.innerHTML=De),Z=o(e),u(v.$$.fragment,e),ee=o(e),x=l(e,"P",{"data-svelte-h":!0}),d(x)!=="svelte-1hcxnu9"&&(x.textContent=Ie),te=o(e),y=l(e,"UL",{"data-svelte-h":!0}),d(y)!=="svelte-p1794t"&&(y.innerHTML=Se),ie=o(e),_=l(e,"P",{"data-svelte-h":!0}),d(_)!=="svelte-1ppoip6"&&(_.innerHTML=Ge),ne=o(e),u(k.$$.fragment,e),ae=o(e),T=l(e,"P",{"data-svelte-h":!0}),d(T)!=="svelte-1p4gi0x"&&(T.textContent=Fe),oe=o(e),$=l(e,"P",{"data-svelte-h":!0}),d($)!=="svelte-22pnmc"&&($.innerHTML=Ue),se=o(e),C=l(e,"P",{"data-svelte-h":!0}),d(C)!=="svelte-16dlqmy"&&(C.innerHTML=je),le=o(e),u(M.$$.fragment,e),de=o(e),L=l(e,"P",{"data-svelte-h":!0}),d(L)!=="svelte-cx8sou"&&(L.textContent=Ee),re=o(e),H=l(e,"P",{"data-svelte-h":!0}),d(H)!=="svelte-1qxkv6z"&&(H.innerHTML=Ae),he=o(e),P=l(e,"P",{"data-svelte-h":!0}),d(P)!=="svelte-1pfro1l"&&(P.textContent=qe),ue=o(e),u(D.$$.fragment,e),fe=o(e),I=l(e,"P",{"data-svelte-h":!0}),d(I)!=="svelte-1kfy227"&&(I.innerHTML=ze),ge=o(e),S=l(e,"P",{"data-svelte-h":!0}),d(S)!=="svelte-ihwxtb"&&(S.innerHTML=Oe),me=o(e),G=l(e,"P",{"data-svelte-h":!0}),d(G)!=="svelte-itjnhm"&&(G.textContent=Ne),ce=o(e),F=l(e,"UL",{"data-svelte-h":!0}),d(F)!=="svelte-ao69km"&&(F.innerHTML=Ke),pe=o(e),u(U.$$.fragment,e),be=o(e),j=l(e,"TABLE",{"data-svelte-h":!0}),d(j)!=="svelte-e3lgjf"&&(j.innerHTML=Re),we=o(e),E=l(e,"P",{"data-svelte-h":!0}),d(E)!=="svelte-6nzchf"&&(E.textContent=We),ve=o(e),A=l(e,"P",{"data-svelte-h":!0}),d(A)!=="svelte-fjxr55"&&(A.innerHTML=Be),xe=o(e),q=l(e,"P",{"data-svelte-h":!0}),d(q)!=="svelte-ys2bcy"&&(q.innerHTML=Ye),ye=o(e),u(z.$$.fragment,e),_e=o(e),O=l(e,"P",{"data-svelte-h":!0}),d(O)!=="svelte-1jxuy2z"&&(O.innerHTML=Qe),ke=o(e),u(N.$$.fragment,e),Te=o(e),K=l(e,"P",{"data-svelte-h":!0}),d(K)!=="svelte-natgq1"&&(K.innerHTML=Ve),$e=o(e),R=l(e,"P",{"data-svelte-h":!0}),d(R)!=="svelte-132h07s"&&(R.innerHTML=Je),Ce=o(e),W=l(e,"P",{"data-svelte-h":!0}),d(W)!=="svelte-rnfmiz"&&(W.innerHTML=Xe),Me=o(e),B=l(e,"P",{"data-svelte-h":!0}),d(B)!=="svelte-1wvf2ag"&&(B.textContent=Ze),Le=o(e),Q=l(e,"P",{}),et(Q).forEach(i),this.h()},h(){tt(r,"name","hf:doc:metadata"),tt(r,"content",ht)},m(e,t){dt(document.head,r),n(e,V,t),n(e,Y,t),n(e,J,t),f(b,e,t),n(e,X,t),n(e,w,t),n(e,Z,t),f(v,e,t),n(e,ee,t),n(e,x,t),n(e,te,t),n(e,y,t),n(e,ie,t),n(e,_,t),n(e,ne,t),f(k,e,t),n(e,ae,t),n(e,T,t),n(e,oe,t),n(e,$,t),n(e,se,t),n(e,C,t),n(e,le,t),f(M,e,t),n(e,de,t),n(e,L,t),n(e,re,t),n(e,H,t),n(e,he,t),n(e,P,t),n(e,ue,t),f(D,e,t),n(e,fe,t),n(e,I,t),n(e,ge,t),n(e,S,t),n(e,me,t),n(e,G,t),n(e,ce,t),n(e,F,t),n(e,pe,t),f(U,e,t),n(e,be,t),n(e,j,t),n(e,we,t),n(e,E,t),n(e,ve,t),n(e,A,t),n(e,xe,t),n(e,q,t),n(e,ye,t),f(z,e,t),n(e,_e,t),n(e,O,t),n(e,ke,t),f(N,e,t),n(e,Te,t),n(e,K,t),n(e,$e,t),n(e,R,t),n(e,Ce,t),n(e,W,t),n(e,Me,t),n(e,B,t),n(e,Le,t),n(e,Q,t),He=!0},p:nt,i(e){He||(g(b.$$.fragment,e),g(v.$$.fragment,e),g(k.$$.fragment,e),g(M.$$.fragment,e),g(D.$$.fragment,e),g(U.$$.fragment,e),g(z.$$.fragment,e),g(N.$$.fragment,e),He=!0)},o(e){m(b.$$.fragment,e),m(v.$$.fragment,e),m(k.$$.fragment,e),m(M.$$.fragment,e),m(D.$$.fragment,e),m(U.$$.fragment,e),m(z.$$.fragment,e),m(N.$$.fragment,e),He=!1},d(e){e&&(i(V),i(Y),i(J),i(X),i(w),i(Z),i(ee),i(x),i(te),i(y),i(ie),i(_),i(ne),i(ae),i(T),i(oe),i($),i(se),i(C),i(le),i(de),i(L),i(re),i(H),i(he),i(P),i(ue),i(fe),i(I),i(ge),i(S),i(me),i(G),i(ce),i(F),i(pe),i(be),i(j),i(we),i(E),i(ve),i(A),i(xe),i(q),i(ye),i(_e),i(O),i(ke),i(Te),i(K),i($e),i(R),i(Ce),i(W),i(Me),i(B),i(Le),i(Q)),i(r),c(b,e),c(v,e),c(k,e),c(M,e),c(D,e),c(U,e),c(z,e),c(N,e)}}}const ht='{"title":"Unit 2: Fine-Tuning, Guidance and Conditioning","local":"unit-2-fine-tuning-guidance-and-conditioning","sections":[{"title":"Start this Unit :rocket:","local":"start-this-unit-rocket","sections":[],"depth":2},{"title":"Fine-Tuning","local":"fine-tuning","sections":[],"depth":2},{"title":"Guidance","local":"guidance","sections":[],"depth":2},{"title":"Conditioning","local":"conditioning","sections":[],"depth":2},{"title":"Hands-On Notebook","local":"hands-on-notebook","sections":[],"depth":2},{"title":"Project Time","local":"project-time","sections":[],"depth":2},{"title":"Some Additional Resources","local":"some-additional-resources","sections":[],"depth":2}],"depth":1}';function ut(Pe){return at(()=>{new URLSearchParams(window.location.search).get("fw")}),[]}class ct extends ot{constructor(r){super(),st(this,r,ut,rt,it,{})}}export{ct as component};
